--
-- Thermal Device
-- 
-- Auther: Bo Wan
-- Created: Feb. 5, 2003
--
-------------------------------------------------------------------------------
-- Description:
-- Verilog-AMS model of the thermal model.
-- Based on Pavil's example
-------------------------------------------------------------------------------

package electrical_system is
	nature electrical is real across real through;
	terminal ground : electrical;
end electrical_system;

package thermal_system is
	nature thermal is real across real through;
	terminal thermal_ground : thermal;
end thermal_system;

use electrical_system.all;
use thermal_system.all;

package thermaldata is
-- CONSTANTS
constant tref : real := 273.15;	-- temperature at which resistance is defined
end package thermaldata;

use thermaldata.all;

entity rhead is
	generic ( 
		r2 : real := 1.0e3;
		alpha : real := 0.1
	);
	port ( 
		terminal pos, neg : electrical;
		terminal t1, t2 : thermal
	);
end rhead;

architecture arch1 of rhead is
	quantity volt across pos to neg;
	quantity curr through pos to neg;
	
	quantity temp across heatflow through t1 to t2;
begin
  Thermal_equations: procedural is
  begin -- procedural statements

	curr := volt / (r2 + alpha * (temp - tref) );
	heatflow := -volt * curr;

  end procedural;
end architecture arch1;
