/**********
Copyright 1990 Regents of the University of California.  All rights reserved.
Author: 1985 Thomas L. Quarles
**********/

/* NIintegrate(ckt,geq,ceq,cap,qcap)
 *  integrate the specified capacitor - method and order in the
 *  ckt structure, ccap follows qcap.
 */

#include "spice.h"
#include <stdio.h>
#include "cktdefs.h"
#include "sperror.h"
#include "util.h"
#include "suffix.h"

#define ccap qcap+1

int
NIintegrate(ckt,geq,ceq,cap,qcap)
    register CKTcircuit *ckt;
    double *geq;
    double *ceq;
    double cap;
    int qcap;
{
    static char *ordmsg = "Illegal integration order";
    static char *methodmsg = "Unknown integration method";

    #ifdef WBPRINT
    printf("Enter NIintegrate\n");
    #endif

    switch(ckt->CKTintegrateMethod) {

    case TRAPEZOIDAL:
        switch(ckt->CKTorder) {
        case 1:
	    #ifdef WBPRINT
	    printf("CKTags:%-.15g\t%-.15g\n",ckt->CKTag[0],ckt->CKTag[1]);
	    printf("CKTstates Before TRAPEZOIDAL: \tCase 1 \n");
	    printf("\t%-.15g\t%-.15g\n",*(ckt->CKTstate0+qcap),*(ckt->CKTstate0+qcap+1));
	    printf("\t%-.15g\t%-.15g\n",*(ckt->CKTstate1+qcap),*(ckt->CKTstate1+qcap+1));
	    printf("\t%-.15g\t%-.15g\n",*(ckt->CKTstate2+qcap),*(ckt->CKTstate2+qcap+1));
	    printf("\t%-.15g\t%-.15g\n",*(ckt->CKTstate3+qcap),*(ckt->CKTstate3+qcap+1));
	    #endif
            *(ckt->CKTstate0+ccap) = ckt->CKTag[0] * (*(ckt->CKTstate0+qcap)) 
                    + ckt->CKTag[1] * (*(ckt->CKTstate1+qcap));
	    #ifdef WBPRINT
	    printf("CKTstates After  TRAPEZOIDAL:\n");
	    printf("\t%-.15g\t%-.15g\n",*(ckt->CKTstate0+qcap),*(ckt->CKTstate0+qcap+1));
	    printf("\t%-.15g\t%-.15g\n",*(ckt->CKTstate1+qcap),*(ckt->CKTstate1+qcap+1));
	    printf("\t%-.15g\t%-.15g\n",*(ckt->CKTstate2+qcap),*(ckt->CKTstate2+qcap+1));
	    printf("\t%-.15g\t%-.15g\n",*(ckt->CKTstate3+qcap),*(ckt->CKTstate3+qcap+1));
	    #endif
            break;
        case 2:
	    #ifdef WBPRINT
	    printf("CKTags:%-.15g\t%-.15g\n",ckt->CKTag[0],ckt->CKTag[1]);
	    printf("CKTstates Before TRAPEZOIDAL: \tCase 2 \n");
	    printf("\t%-.15g\t%-.15g\n",*(ckt->CKTstate0+qcap),*(ckt->CKTstate0+qcap+1));
	    printf("\t%-.15g\t%-.15g\n",*(ckt->CKTstate1+qcap),*(ckt->CKTstate1+qcap+1));
	    printf("\t%-.15g\t%-.15g\n",*(ckt->CKTstate2+qcap),*(ckt->CKTstate2+qcap+1));
	    printf("\t%-.15g\t%-.15g\n",*(ckt->CKTstate3+qcap),*(ckt->CKTstate3+qcap+1));
	    #endif
            *(ckt->CKTstate0+ccap) = - *(ckt->CKTstate1+ccap) * ckt->CKTag[1] + 
                    ckt->CKTag[0] * 
                    ( *(ckt->CKTstate0+qcap) - *(ckt->CKTstate1+qcap) );
	    #ifdef WBPRINT
	    printf("CKTstates After  TRAPEZOIDAL:\n");
	    printf("\t%-.15g\t%-.15g\n",*(ckt->CKTstate0+qcap),*(ckt->CKTstate0+qcap+1));
	    printf("\t%-.15g\t%-.15g\n",*(ckt->CKTstate1+qcap),*(ckt->CKTstate1+qcap+1));
	    printf("\t%-.15g\t%-.15g\n",*(ckt->CKTstate2+qcap),*(ckt->CKTstate2+qcap+1));
	    printf("\t%-.15g\t%-.15g\n",*(ckt->CKTstate3+qcap),*(ckt->CKTstate3+qcap+1));
	    #endif
            break;
        default:
            errMsg = MALLOC(strlen(ordmsg)+1);
            strcpy(errMsg,ordmsg);
            return(E_ORDER);
        }
        break;
    case GEAR:
	    #ifdef WBPRINT
		printf("GEAR, CKTorder: %d\n",ckt->CKTorder);
		#endif
        *(ckt->CKTstate0+ccap)=0;
        switch(ckt->CKTorder) {

        case 6:
            *(ckt->CKTstate0+ccap) += ckt->CKTag[6]* *(ckt->CKTstate6+qcap);
            /* fall through */
        case 5:
            *(ckt->CKTstate0+ccap) += ckt->CKTag[5]* *(ckt->CKTstate5+qcap);
            /* fall through */
        case 4:
            *(ckt->CKTstate0+ccap) += ckt->CKTag[4]* *(ckt->CKTstate4+qcap);
            /* fall through */
        case 3:
            *(ckt->CKTstate0+ccap) += ckt->CKTag[3]* *(ckt->CKTstate3+qcap);
            /* fall through */
        case 2:
            *(ckt->CKTstate0+ccap) += ckt->CKTag[2]* *(ckt->CKTstate2+qcap);
            /* fall through */
        case 1:
            *(ckt->CKTstate0+ccap) += ckt->CKTag[1]* *(ckt->CKTstate1+qcap);
            *(ckt->CKTstate0+ccap) += ckt->CKTag[0]* *(ckt->CKTstate0+qcap);
            break;

        default:
            return(E_ORDER);

        }
        break;

    default:
        errMsg = MALLOC(strlen(methodmsg)+1);
        strcpy(errMsg,methodmsg);
        return(E_METHOD);
    }
    *ceq = *(ckt->CKTstate0+ccap) - ckt->CKTag[0] * *(ckt->CKTstate0+qcap);
    *geq = ckt->CKTag[0] * cap;
    #ifdef WBPRINT
    printf("ceq & geq:%-.15g\t%-.15g\n",*ceq,*geq);
    #endif
    return(OK);
}
